@@ -33,35 +33,47 @@
 #define RC_LEVEL_BOOT           "boot"
 #define RC_LEVEL_DEFAULT        "default"
 
-#define RC_DEPTREE_CACHE        RC_SVCDIR "/deptree"
-#define RC_DEPTREE_SKEWED	RC_SVCDIR "/clock-skewed"
 #define RC_KRUNLEVEL            RC_SVCDIR "/krunlevel"
-#define RC_STARTING             RC_SVCDIR "/rc.starting"
-#define RC_STOPPING             RC_SVCDIR "/rc.stopping"
-
-#define RC_SVCDIR_STARTING      RC_SVCDIR "/starting"
-#define RC_SVCDIR_INACTIVE      RC_SVCDIR "/inactive"
-#define RC_SVCDIR_STARTED       RC_SVCDIR "/started"
-#define RC_SVCDIR_COLDPLUGGED	RC_SVCDIR "/coldplugged"
 
 char *rc_conf_value(const char *var);
 bool rc_conf_yesno(const char *var);
 void env_filter(void);
 void env_config(void);
 int signal_setup(int sig, void (*handler)(int));
-int signal_setup_restart(int sig, void (*handler)(int));
 int svc_lock(const char *, bool);
 int svc_unlock(const char *, int);
 pid_t exec_service(const char *, const char *);
 
+typedef struct rc_service_state_name {
+	RC_SERVICE state;
+	enum rc_dir dir;
+	const char *const name;
+} rc_service_state_name_t;
+
+/* We MUST list the states below 0x10 first
+ * The rest can be in any order */
+static const rc_service_state_name_t rc_service_state_names[] = {
+	{ RC_SERVICE_STARTED,     RC_DIR_STARTED,     "started" },
+	{ RC_SERVICE_STOPPED,     RC_DIR_INVALID,     "stopped" },
+	{ RC_SERVICE_STARTING,    RC_DIR_STARTING,    "starting" },
+	{ RC_SERVICE_STOPPING,    RC_DIR_STOPPING,    "stopping" },
+	{ RC_SERVICE_INACTIVE,    RC_DIR_INACTIVE,    "inactive" },
+	{ RC_SERVICE_WASINACTIVE, RC_DIR_WASINACTIVE, "wasinactive" },
+	{ RC_SERVICE_HOTPLUGGED,  RC_DIR_HOTPLUGGED,  "hotplugged" },
+	{ RC_SERVICE_FAILED,      RC_DIR_FAILED,      "failed" },
+	{ RC_SERVICE_SCHEDULED,   RC_DIR_SCHEDULED,   "scheduled"},
+	{ RC_SERVICE_CRASHED,     RC_DIR_INVALID,     "crashed"},
+	{ 0, -1, NULL}
+};
+
 /*
  * Check whether path is writable or not,
  * this also works properly with read-only filesystems
  */
 int is_writable(const char *);
 
-#define service_start(service) exec_service(service, "start");
-#define service_stop(service)  exec_service(service, "stop");
+#define service_start(service) exec_service(service, "start")
+#define service_stop(service)  exec_service(service, "stop")
 
 int parse_mode(mode_t *, char *);
 
@@ -73,4 +85,21 @@ void from_time_t(char *time_string, time_t tv);
 time_t to_time_t(char *timestring);
 pid_t get_pid(const char *applet, const char *pidfile);
 
+void cloexec_fds_from(int);
+
+struct notify {
+	enum {
+		NOTIFY_NONE = 0,
+		NOTIFY_FD,
+		NOTIFY_SOCKET
+	} type;
+
+	char *path;
+	int pipe[2];
+	int fd;
+};
+
+struct notify notify_parse(const char *applet, const char *ready_string);
+bool notify_wait(const char *applet, struct notify ready);
+
 #endif
